==> openturns_0.12.2.orig.tar.gz: openturns-0.12.2/lib/src/Uncertainty/Algorithm/IsoProbabilisticTransformation/NatafIndependentCopula/NatafIndependentCopulaHessian.cxx <==
//                                               -*- C++ -*-
/**
 *  @file  NatafIndependentCopulaHessian.cxx
 *  @brief Class for the Nataf transformation evaluation for independent
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2008-10-31 11:52:04 +0100 (ven 31 oct 2008) $
 *  Id:      $Id: NatafIndependentCopulaHessian.cxx 995 2008-10-31 10:52:04Z dutka $
 */
#include <cmath>
#include "NatafIndependentCopulaHessian.hxx"
#include "DistFunc.hxx"

namespace OpenTURNS {

  namespace Uncertainty {

    namespace Algorithm {

      /*
       * @class NatafIndependentCopulaHessian
       *
       * This class offers an interface for the Nataf function for independent copula
       */

      typedef Distribution::DistFunc DistFunc;

      CLASSNAMEINIT(NatafIndependentCopulaHessian);

      /* Parameter constructor */
      NatafIndependentCopulaHessian::NatafIndependentCopulaHessian(const UnsignedLong dimension):
	NumericalMathHessianImplementation(),
	dimension_(dimension)
      {
	// Nothing to do
      }

      /* Virtual constructor */
      NatafIndependentCopulaHessian * NatafIndependentCopulaHessian::clone() const
      {
	return new NatafIndependentCopulaHessian(*this);
      }

      /* String converter */
      String NatafIndependentCopulaHessian::str() const
      {
	OSS oss;
	oss << "class=" << NatafIndependentCopulaHessian::GetClassName()
            << " dimension=" << dimension_;
	return oss;
      }

      /*
       * Evaluation
       * The Nataf transform T reads:
       * Ti(xi) = Q(xi), where Q = Phi^{-1} and Phi is the CDF of the standard normal distribution
       * We have:
       * Hijk = d2Ti/dxjdxk = Q''(xi) if i = j = k
       *                    = 0 else
       */
      NatafIndependentCopulaHessian::SymmetricTensor NatafIndependentCopulaHessian::hessian(const NumericalPoint & in) const
	throw(InvalidArgumentException, InternalException)
      {
	SymmetricTensor result(dimension_, dimension_);
	for (UnsignedLong i = 0; i < dimension_; ++i)
	  {
	    NumericalScalar x(in[i]);
	    if ((x <= 0.0) || (x >= 1.0)) throw InvalidArgumentException(HERE) << "Error: cannot evaluate the NatafIndependentCopulaHessian if all the components are not in ]0, 1[, here in=" << in;
	    // q = Normal(0,1).computeQuantile(x)
	    NumericalScalar q(DistFunc::qNormal(x));
	    // factor = 1/Normal(0,1).computePDF(q)
	    // 6.283185307179586476925286 = 2Pi
	    // quantileSecondDerivative = -Normal(0,1).computeDDF(q) / (Normal(0,1).computePDF(q))^3 = q / (Normal(0,1).computePDF(q))^2
	    result(i, i, i) = 6.283185307179586476925286 * q * exp(q * q);
	  }
	return result;
      }

      /* Accessor for input point dimension */
      UnsignedLong NatafIndependentCopulaHessian::getInputNumericalPointDimension() const
	throw(InternalException)
      {
	return dimension_;
      }

      /* Accessor for output point dimension */
      UnsignedLong NatafIndependentCopulaHessian::getOutputNumericalPointDimension() const
	throw(InternalException)
      {
	return dimension_;
      }

    } /* namespace Algorithm */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */

==> openturns_0.13.2.orig.tar.gz: openturns-0.13.2/lib/src/Uncertainty/Algorithm/IsoProbabilisticTransformation/NatafIndependentCopula/NatafIndependentCopulaHessian.cxx <==
//                                               -*- C++ -*-
/**
 *  @file  NatafIndependentCopulaHessian.cxx
 *  @brief Class for the Nataf transformation evaluation for independent
 *
 *  (C) Copyright 2005-2010 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2010-02-04 16:44:49 +0100 (jeu. 04 févr. 2010) $
 *  Id:      $Id: NatafIndependentCopulaHessian.cxx 1473 2010-02-04 15:44:49Z dutka $
 */
#include <cmath>
#include "NatafIndependentCopulaHessian.hxx"
#include "DistFunc.hxx"

namespace OpenTURNS {

  namespace Uncertainty {

    namespace Algorithm {

      /*
       * @class NatafIndependentCopulaHessian
       *
       * This class offers an interface for the Nataf function for independent copula
       */

      typedef Distribution::DistFunc DistFunc;

      CLASSNAMEINIT(NatafIndependentCopulaHessian);

      /* Parameter constructor */
      NatafIndependentCopulaHessian::NatafIndependentCopulaHessian(const UnsignedLong dimension):
	NumericalMathHessianImplementation(),
	dimension_(dimension)
      {
	// Nothing to do
      }

      /* Virtual constructor */
      NatafIndependentCopulaHessian * NatafIndependentCopulaHessian::clone() const
      {
	return new NatafIndependentCopulaHessian(*this);
      }

      /* String converter */
      String NatafIndependentCopulaHessian::__repr__() const
      {
	OSS oss;
	oss << "class=" << NatafIndependentCopulaHessian::GetClassName()
            << " dimension=" << dimension_;
	return oss;
      }

      /*
       * Evaluation
       * The Nataf transform T reads:
       * Ti(xi) = Q(xi), where Q = Phi^{-1} and Phi is the CDF of the standard normal distribution
       * We have:
       * Hijk = d2Ti/dxjdxk = Q''(xi) if i = j = k
       *                    = 0 else
       */
      NatafIndependentCopulaHessian::SymmetricTensor NatafIndependentCopulaHessian::hessian(const NumericalPoint & in) const
	/* throw(InvalidArgumentException, InternalException) */
      {
	SymmetricTensor result(dimension_, dimension_);
	for (UnsignedLong i = 0; i < dimension_; ++i)
	  {
	    NumericalScalar x(in[i]);
	    if ((x < 0.0) || (x > 1.0)) throw InvalidArgumentException(HERE) << "Error: cannot evaluate the NatafIndependentCopulaHessian if all the components are not in [0, 1], here in=" << in;
	    // q = Normal(0,1).computeQuantile(x)
	    NumericalScalar q(DistFunc::qNormal(x));
	    // factor = 1/Normal(0,1).computePDF(q)
	    // 6.283185307179586476925286 = 2Pi
	    // quantileSecondDerivative = -Normal(0,1).computeDDF(q) / (Normal(0,1).computePDF(q))^3 = q / (Normal(0,1).computePDF(q))^2
	    result(i, i, i) = 6.283185307179586476925286 * q * exp(q * q);
	  }
	return result;
      }

      /* Accessor for input point dimension */
      UnsignedLong NatafIndependentCopulaHessian::getInputDimension() const
	/* throw(InternalException) */
      {
	return dimension_;
      }

      /* Accessor for output point dimension */
      UnsignedLong NatafIndependentCopulaHessian::getOutputDimension() const
	/* throw(InternalException) */
      {
	return dimension_;
      }

    } /* namespace Algorithm */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */
